@@ -35,7 +35,7 @@ import os.path
 
 import salt.utils.args
 import salt.utils.compat
-from salt.utils.versions import LooseVersion as _LooseVersion
+from salt.utils.versions import Version
 
 log = logging.getLogger(__name__)
 
@@ -43,13 +43,13 @@ REQUIRED_LIBCLOUD_VERSION = "2.0.0"
 try:
     # pylint: disable=unused-import
     import libcloud
-    from libcloud.compute.providers import get_driver
     from libcloud.compute.base import Node
+    from libcloud.compute.providers import get_driver
 
     # pylint: enable=unused-import
-    if hasattr(libcloud, "__version__") and _LooseVersion(
-        libcloud.__version__
-    ) < _LooseVersion(REQUIRED_LIBCLOUD_VERSION):
+    if hasattr(libcloud, "__version__") and Version(libcloud.__version__) < Version(
+        REQUIRED_LIBCLOUD_VERSION
+    ):
         raise ImportError()
     logging.getLogger("libcloud").setLevel(logging.CRITICAL)
     HAS_LIBCLOUD = True
@@ -62,10 +62,12 @@ def __virtual__():
     Only load if libcloud libraries exist.
     """
     if not HAS_LIBCLOUD:
-        msg = (
-            "A apache-libcloud library with version at least {} was not " "found"
-        ).format(REQUIRED_LIBCLOUD_VERSION)
-        return (False, msg)
+        return (
+            False,
+            "A apache-libcloud library with version at least {} was not found".format(
+                REQUIRED_LIBCLOUD_VERSION
+            ),
+        )
     return True
 
 
