@@ -328,11 +328,12 @@ def wait(
     shell=None,
     env=(),
     stateful=False,
-    umask=None,
     output_loglevel="debug",
     hide_output=False,
     use_vt=False,
     success_retcodes=None,
+    success_stdout=None,
+    success_stderr=None,
     **kwargs
 ):
     """
@@ -402,13 +403,10 @@ def wait(
                   - PATH: {{ [current_path, '/my/special/bin']|join(':') }}
 
         .. note::
-            When using environment variables on Window's, case-sensitivity
-            matters, i.e. Window's uses `Path` as opposed to `PATH` for other
+            When using environment variables on Windows, case-sensitivity
+            matters, i.e. Windows uses `Path` as opposed to `PATH` for other
             systems.
 
-    umask
-         The umask (in octal) to use when running the command.
-
     stateful
         The command being executed is expected to return data about executing
         a state. For more information, see the :ref:`stateful-argument` section.
@@ -442,12 +440,27 @@ def wait(
         interactively to the console and the logs.
         This is experimental.
 
-    success_retcodes: This parameter will be allow a list of
-        non-zero return codes that should be considered a success.  If the
-        return code returned from the run matches any in the provided list,
-        the return code will be overridden with zero.
+    success_retcodes
+        This parameter allows you to specify a list of non-zero return codes
+        that should be considered as successful. If the return code from the
+        command matches any in the list, the state will have a ``True`` result
+        instead of ``False``.
 
       .. versionadded:: 2019.2.0
+
+    success_stdout: This parameter will allow a list of
+        strings that when found in standard out should be considered a success.
+        If stdout returned from the run matches any in the provided list,
+        the return code will be overridden with zero.
+
+      .. versionadded:: 3004
+
+    success_stderr: This parameter will allow a list of
+        strings that when found in standard error should be considered a success.
+        If stderr returned from the run matches any in the provided list,
+        the return code will be overridden with zero.
+
+      .. versionadded:: 3004
     """
     # Ignoring our arguments is intentional.
     return {"name": name, "changes": {}, "result": True, "comment": ""}
@@ -466,10 +479,12 @@ def wait_script(
     shell=None,
     env=None,
     stateful=False,
-    umask=None,
     use_vt=False,
     output_loglevel="debug",
     hide_output=False,
+    success_retcodes=None,
+    success_stdout=None,
+    success_stderr=None,
     **kwargs
 ):
     """
@@ -542,13 +557,10 @@ def wait_script(
                   - PATH: {{ [current_path, '/my/special/bin']|join(':') }}
 
         .. note::
-            When using environment variables on Window's, case-sensitivity
-            matters, i.e. Window's uses `Path` as opposed to `PATH` for other
+            When using environment variables on Windows, case-sensitivity
+            matters, i.e. Windows uses `Path` as opposed to `PATH` for other
             systems.
 
-    umask
-         The umask (in octal) to use when running the command.
-
     stateful
         The command being executed is expected to return data about executing
         a state. For more information, see the :ref:`stateful-argument` section.
@@ -575,12 +587,27 @@ def wait_script(
 
         .. versionadded:: 2018.3.0
 
-    success_retcodes: This parameter will be allow a list of
-        non-zero return codes that should be considered a success.  If the
-        return code returned from the run matches any in the provided list,
-        the return code will be overridden with zero.
+    success_retcodes
+        This parameter allows you to specify a list of non-zero return codes
+        that should be considered as successful. If the return code from the
+        command matches any in the list, the state will have a ``True`` result
+        instead of ``False``.
 
       .. versionadded:: 2019.2.0
+
+    success_stdout: This parameter will allow a list of
+        strings that when found in standard out should be considered a success.
+        If stdout returned from the run matches any in the provided list,
+        the return code will be overridden with zero.
+
+      .. versionadded:: 3004
+
+    success_stderr: This parameter will allow a list of
+        strings that when found in standard error should be considered a success.
+        If stderr returned from the run matches any in the provided list,
+        the return code will be overridden with zero.
+
+      .. versionadded:: 3004
     """
     # Ignoring our arguments is intentional.
     return {"name": name, "changes": {}, "result": True, "comment": ""}
@@ -595,19 +622,31 @@ def run(
     env=None,
     prepend_path=None,
     stateful=False,
-    umask=None,
     output_loglevel="debug",
     hide_output=False,
     timeout=None,
     ignore_timeout=False,
     use_vt=False,
     success_retcodes=None,
+    success_stdout=None,
+    success_stderr=None,
     **kwargs
 ):
     """
     Run a command if certain circumstances are met.  Use ``cmd.wait`` if you
     want to use the ``watch`` requisite.
 
+    .. note::
+
+       The ``**kwargs`` of ``cmd.run`` are passed down to one of the following
+       exec modules:
+
+       * ``cmdmod.run_all``: If used with default ``runas``
+       * ``cmdmod.run_chroot``: If used with non-``root`` value for ``runas``
+
+       For more information on what args are available for either of these,
+       refer to the :ref:`cmdmod documentation <cmdmod-module>`.
+
     name
         The command to execute, remember that the command will execute with the
         path and permissions of the salt-minion.
@@ -621,7 +660,7 @@ def run(
         will run inside a chroot
 
     runas
-        The user name to run the command as
+        The user name (or uid) to run the command as
 
     shell
         The shell to use for execution, defaults to the shell grain
@@ -667,8 +706,8 @@ def run(
                   - PATH: {{ [current_path, '/my/special/bin']|join(':') }}
 
         .. note::
-            When using environment variables on Window's, case-sensitivity
-            matters, i.e. Window's uses `Path` as opposed to `PATH` for other
+            When using environment variables on Windows, case-sensitivity
+            matters, i.e. Windows uses `Path` as opposed to `PATH` for other
             systems.
 
     prepend_path
@@ -681,9 +720,6 @@ def run(
         The command being executed is expected to return data about executing
         a state. For more information, see the :ref:`stateful-argument` section.
 
-    umask
-        The umask (in octal) to use when running the command.
-
     output_loglevel : debug
         Control the loglevel at which the output from the command is logged to
         the minion log.
@@ -729,13 +765,28 @@ def run(
 
         .. versionadded:: 2016.3.6
 
-    success_retcodes: This parameter will be allow a list of
-        non-zero return codes that should be considered a success.  If the
-        return code returned from the run matches any in the provided list,
-        the return code will be overridden with zero.
+    success_retcodes
+        This parameter allows you to specify a list of non-zero return codes
+        that should be considered as successful. If the return code from the
+        command matches any in the list, the state will have a ``True`` result
+        instead of ``False``.
 
       .. versionadded:: 2019.2.0
 
+    success_stdout: This parameter will allow a list of
+        strings that when found in standard out should be considered a success.
+        If stdout returned from the run matches any in the provided list,
+        the return code will be overridden with zero.
+
+      .. versionadded:: 3004
+
+    success_stderr: This parameter will allow a list of
+        strings that when found in standard error should be considered a success.
+        If stderr returned from the run matches any in the provided list,
+        the return code will be overridden with zero.
+
+      .. versionadded:: 3004
+
     .. note::
 
         cmd.run supports the usage of ``reload_modules``. This functionality
@@ -774,7 +825,7 @@ def run(
     # Need the check for None here, if env is not provided then it falls back
     # to None and it is assumed that the environment is not being overridden.
     if env is not None and not isinstance(env, (list, dict)):
-        ret["comment"] = "Invalidly-formatted 'env' parameter. See " "documentation."
+        ret["comment"] = "Invalidly-formatted 'env' parameter. See documentation."
         return ret
 
     cmd_kwargs = copy.deepcopy(kwargs)
@@ -787,22 +838,22 @@ def run(
             "shell": shell or __grains__["shell"],
             "env": env,
             "prepend_path": prepend_path,
-            "umask": umask,
             "output_loglevel": output_loglevel,
             "hide_output": hide_output,
             "success_retcodes": success_retcodes,
+            "success_stdout": success_stdout,
+            "success_stderr": success_stderr,
         }
     )
 
     if __opts__["test"] and not test_name:
         ret["result"] = None
         ret["comment"] = 'Command "{}" would have been executed'.format(name)
+        ret["changes"] = {"cmd": name}
         return _reinterpreted_state(ret) if stateful else ret
 
     if cwd and not os.path.isdir(cwd):
-        ret["comment"] = ('Desired working directory "{}" ' "is not available").format(
-            cwd
-        )
+        ret["comment"] = 'Desired working directory "{}" is not available'.format(cwd)
         return ret
 
     # Wow, we passed the test, run this sucker!
@@ -845,7 +896,6 @@ def script(
     shell=None,
     env=None,
     stateful=False,
-    umask=None,
     timeout=None,
     use_vt=False,
     output_loglevel="debug",
@@ -853,6 +903,8 @@ def script(
     defaults=None,
     context=None,
     success_retcodes=None,
+    success_stdout=None,
+    success_stderr=None,
     **kwargs
 ):
     """
@@ -884,7 +936,7 @@ def script(
 
         .. note::
 
-            For Window's users, specifically Server users, it may be necessary
+            For Windows users, specifically Server users, it may be necessary
             to specify your runas user using the User Logon Name instead of the
             legacy logon name. Traditionally, logons would be in the following
             format.
@@ -949,16 +1001,13 @@ def script(
                   - PATH: {{ [current_path, '/my/special/bin']|join(':') }}
 
         .. note::
-            When using environment variables on Window's, case-sensitivity
-            matters, i.e. Window's uses `Path` as opposed to `PATH` for other
+            When using environment variables on Windows, case-sensitivity
+            matters, i.e. Windows uses `Path` as opposed to `PATH` for other
             systems.
 
     saltenv : ``base``
         The Salt environment to use
 
-    umask
-         The umask (in octal) to use when running the command.
-
     stateful
         The command being executed is expected to return data about executing
         a state. For more information, see the :ref:`stateful-argument` section.
@@ -1012,13 +1061,27 @@ def script(
 
         .. versionadded:: 2018.3.0
 
-    success_retcodes: This parameter will be allow a list of
-        non-zero return codes that should be considered a success.  If the
-        return code returned from the run matches any in the provided list,
-        the return code will be overridden with zero.
+    success_retcodes
+        This parameter allows you to specify a list of non-zero return codes
+        that should be considered as successful. If the return code from the
+        command matches any in the list, the state will have a ``True`` result
+        instead of ``False``.
 
       .. versionadded:: 2019.2.0
 
+    success_stdout: This parameter will allow a list of
+        strings that when found in standard out should be considered a success.
+        If stdout returned from the run matches any in the provided list,
+        the return code will be overridden with zero.
+
+      .. versionadded:: 3004
+
+    success_stderr: This parameter will allow a list of
+        strings that when found in standard error should be considered a success.
+        If stderr returned from the run matches any in the provided list,
+        the return code will be overridden with zero.
+
+      .. versionadded:: 3004
     """
     test_name = None
     if not isinstance(stateful, list):
@@ -1033,22 +1096,22 @@ def script(
     # Need the check for None here, if env is not provided then it falls back
     # to None and it is assumed that the environment is not being overridden.
     if env is not None and not isinstance(env, (list, dict)):
-        ret["comment"] = "Invalidly-formatted 'env' parameter. See " "documentation."
+        ret["comment"] = "Invalidly-formatted 'env' parameter. See documentation."
         return ret
 
     if context and not isinstance(context, dict):
-        ret["comment"] = (
-            "Invalidly-formatted 'context' parameter. Must " "be formed as a dict."
-        )
+        ret[
+            "comment"
+        ] = "Invalidly-formatted 'context' parameter. Must be formed as a dict."
         return ret
     if defaults and not isinstance(defaults, dict):
-        ret["comment"] = (
-            "Invalidly-formatted 'defaults' parameter. Must " "be formed as a dict."
-        )
+        ret[
+            "comment"
+        ] = "Invalidly-formatted 'defaults' parameter. Must be formed as a dict."
         return ret
 
     if runas and salt.utils.platform.is_windows() and not password:
-        ret["commnd"] = "Must supply a password if runas argument is used on Windows."
+        ret["comment"] = "Must supply a password if runas argument is used on Windows."
         return ret
 
     tmpctx = defaults if defaults else {}
@@ -1064,7 +1127,6 @@ def script(
             "env": env,
             "cwd": cwd,
             "template": template,
-            "umask": umask,
             "timeout": timeout,
             "output_loglevel": output_loglevel,
             "hide_output": hide_output,
@@ -1072,6 +1134,8 @@ def script(
             "context": tmpctx,
             "saltenv": __env__,
             "success_retcodes": success_retcodes,
+            "success_stdout": success_stdout,
+            "success_stderr": success_stderr,
         }
     )
 
@@ -1091,13 +1155,11 @@ def script(
 
     if __opts__["test"] and not test_name:
         ret["result"] = None
-        ret["comment"] = "Command '{}' would have been " "executed".format(name)
+        ret["comment"] = "Command '{}' would have been executed".format(name)
         return _reinterpreted_state(ret) if stateful else ret
 
     if cwd and not os.path.isdir(cwd):
-        ret["comment"] = ('Desired working directory "{}" ' "is not available").format(
-            cwd
-        )
+        ret["comment"] = 'Desired working directory "{}" is not available'.format(cwd)
         return ret
 
     # Wow, we passed the test, run this sucker!
@@ -1113,7 +1175,7 @@ def script(
     else:
         ret["result"] = not bool(cmd_all["retcode"])
     if ret.get("changes", {}).get("cache_error"):
-        ret["comment"] = "Unable to cache script {} from saltenv " "'{}'".format(
+        ret["comment"] = "Unable to cache script {} from saltenv '{}'".format(
             source, __env__
         )
     else:
@@ -1157,7 +1219,7 @@ def call(
             'name': name
             'changes': {'retval': result},
             'result': True if result is None else bool(result),
-            'comment': result if isinstance(result, six.string_types) else ''
+            'comment': result if isinstance(result, str) else ''
         }
     """
     ret = {"name": name, "changes": {}, "result": False, "comment": ""}
@@ -1170,7 +1232,6 @@ def call(
         "use_vt": use_vt,
         "output_loglevel": output_loglevel,
         "hide_output": hide_output,
-        "umask": kwargs.get("umask"),
     }
 
     if not kws:
@@ -1233,16 +1294,16 @@ def mod_watch(name, **kwargs):
             return {
                 "name": name,
                 "changes": {},
-                "comment": ("cmd.{0[sfun]} needs a named parameter func").format(
-                    kwargs
-                ),
+                "comment": "cmd.{0[sfun]} needs a named parameter func".format(kwargs),
                 "result": False,
             }
 
     return {
         "name": name,
         "changes": {},
-        "comment": "cmd.{0[sfun]} does not work with the watch requisite, "
-        "please use cmd.wait or cmd.wait_script".format(kwargs),
+        "comment": (
+            "cmd.{0[sfun]} does not work with the watch requisite, "
+            "please use cmd.wait or cmd.wait_script".format(kwargs)
+        ),
         "result": False,
     }
